"""
Base tests.
"""

import logging

from django.test import TransactionTestCase
import pytest

import factories

__title__ = 'strawberry.tests.base'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2018 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'BaseTestCase',
)

LOGGER = logging.getLogger(__name__)


@pytest.mark.django_db
class BaseTestCase(TransactionTestCase):
    """Base test case."""

    pytestmark = pytest.mark.django_db

    @classmethod
    def setUpClass(cls):
        """Set up class."""

        # Create test user.
        cls.user = factories.TestUsernameSuperAdminUserFactory()



# -*- coding: utf-8 -*-

import hashlib

__title__ = 'strawberry.fields'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2018 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'get_prepopulated_value',
    'generate_md5_hash',
)


def get_prepopulated_value(field, instance):
    """Returns preliminary value based on `populate_from`.

    Args:
        field (:obj:`strawberry.fields.md5.MD5Field`): `MD5Field` instance.
        instance (:obj:`django.db.models.Model`): Django model instance.

    Returns:
        str: Prepopulated value.
    """
    if callable(field.populate_from):
        # import pytest; pytest.set_trace()
        # MD5Field(populate_from=lambda instance: ...)
        return field.populate_from(instance)
    else:
        # MD5Field(populate_from='foo')
        attr = getattr(instance, field.populate_from)
        return callable(attr) and attr() or attr


def generate_md5_hash(value, unique=False):
    """Generate md5 hash.

    Args:
        value (str):
        unique (bool):

    Returns:
        str: Generated MD5 hash.
    """
    md5 = hashlib.md5()
    encoded_value = value.encode('utf8')
    md5.update(encoded_value)
    return md5.hexdigest()



"""
Tests.
"""
from .test_fields import TestMD5Field

__title__ = 'strawberry.tests'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2018 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'TestMD5Field',
)



# -*- coding: utf-8 -*-
"""
Test fields.
"""

from __future__ import absolute_import, unicode_literals

import hashlib
import unittest

import pytest

import factories

from .base import BaseTestCase

__title__ = 'strawberry.tests.test_fields'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2018 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'TestMD5Field',
)


BOOK_DATA = {
    'title': "CHAPTER II. The Pool of Tears.",
    'summary': "Alice spots another creature in the pool, swimming far "
               "off. She sees that it is a mouse, who has also slipped "
               "into the pool of tears. Alice thinks she might as well "
               "try speaking to the mouse but he doesn't seem to "
               "understand English, so she tries addressing him in "
               "French. The first phrase she thinks of is “Ou est ma "
               "chatte?“ which means “Where is my cat?“ The mouse is "
               "suitably unnerved. Alice protests that the mouse would "
               "like her cat, Dinah, and proceeds to list her virtues. "
               "The mouse is very offended.",
    'description': "As she said these words her foot slipped, and in "
                   "another moment, splash! she was up to her chin in "
                   "salt water. Her first idea was that she had somehow "
                   "fallen into the sea, ‘and in that case I can go back "
                   "by railway,’ she said to herself. (Alice had been to "
                   "the seaside once in her life, and had come to the "
                   "general conclusion, that wherever you go to on the "
                   "English coast you find a number of bathing machines "
                   "in the sea, some children digging in the sand with "
                   "wooden spades, then a row of lodging houses, and "
                   "behind them a railway station.) However, she soon "
                   "made out that she was in the pool of tears which she "
                   "had wept when she was nine feet high.",
    'credits': """
               Charles Lutwidge Dodgson under the pseudonym Lewis Carroll.
               """,
    'synopsis': """
                Chapter Two – The Pool of Tears: Chapter Two opens with Alice   
                growing to such a tremendous size her head hits the ceiling.   
                Alice is unhappy and, as she cries, her tears flood the   
                hallway. After shrinking down again due to a fan she had picked    
                up, Alice swims through her own tears and meets a Mouse, who is    
                swimming as well. She tries to make small talk with him in    
                elementary French (thinking he may be a French mouse) but her    
                opening gambit "Où est ma chatte?" ("Where is my cat?") offends 
                the mouse and he tries to escape her.
                """,
}

BOOK_DATA_UPDATE = {
    'title': "CHAPTER II. Summary and description.",
    'summary': "‘What a curious feeling!’ said Alice; ‘I must be shutting "
               "up like a telescope.’ However, she soon made out that she "
               "was in the pool of tears which she had wept when she was "
               "nine feet high.",
    'description': "So she was considering in her own mind (as well as "
                   "she could, for the hot day made her feel very sleepy "
                   "and stupid), whether the pleasure of making a "
                   "daisy-chain would be worth the trouble of getting up "
                   "and picking the daisies, when suddenly a White Rabbit "
                   "with pink eyes ran close by her. However, she soon "
                   "made out that she was in the pool of tears which she "
                   "had wept when she was nine feet high.",
    'credits': """
               Lewis Carroll (real name Charles Lutwidge Dodgson).
               """,
    'synopsis': """
                Chapter One – Down the Rabbit Hole: Alice, a girl of seven 
                years, is feeling bored and drowsy while sitting on the 
                riverbank with her elder sister. She then notices a talking, 
                clothed White Rabbit with a pocket watch run past. She follows 
                it down a rabbit hole when suddenly she falls a long way to a 
                curious hall with many locked doors of all sizes. She finds a 
                small key to a door too small for her to fit through, but 
                through it she sees an attractive garden. She then discovers 
                a bottle on a table labelled "DRINK ME," the contents of which 
                cause her to shrink too small to reach the key which she has 
                left on the table. She eats a cake with "EAT ME" written on 
                it in currants as the chapter closes.
                """

}


@pytest.mark.django_db
class TestMD5Field(BaseTestCase):
    """Test MD5 field.

    - description_hash is nullable, auto-updated field, with auto whitespace
      strip.
    - summary_hash is nullable, auto-updated field, with auto whitespace
      strip.
    - credits_hash is non-nullable, auto-updated field.
    - synopsis_hash is non-nullable, auto-updated field, with manual
      whitespace strip.
    """

    pytestmark = pytest.mark.django_db

    @classmethod
    def setUpClass(cls):
        pass

    def setUp(self):
        self.book = factories.BookFactory(**BOOK_DATA)

    def _test_filter_by_field(self, model_hash_field, non_hashed_value):
        """Test filter by field.

        Args:
            model_hash_field:
            non_hashed_value:

        Returns:

        """
        md5 = hashlib.md5()
        text = non_hashed_value.encode('utf8')
        md5.update(text)
        self.assertEqual(
            model_hash_field,
            md5.hexdigest()
        )

    def test_filter_by_field(self):
        """Filter by field."""
        self._test_filter_by_field(
            self.book.summary_hash,
            BOOK_DATA['summary']
        )

        self._test_filter_by_field(
            self.book.description_hash,
            BOOK_DATA['description']
        )

        self._test_filter_by_field(
            self.book.credits_hash,
            BOOK_DATA['credits']
        )

        self._test_filter_by_field(
            self.book.synopsis_hash,
            BOOK_DATA['synopsis'].strip()
        )

    def test_filter_by_field_check_update(self):
        """Filter by field, check value update."""
        self.book.summary = BOOK_DATA_UPDATE['summary']
        self.book.description = BOOK_DATA_UPDATE['description']
        self.book.credits = BOOK_DATA_UPDATE['credits']
        self.book.synopsis = BOOK_DATA_UPDATE['synopsis']
        self.book.save()

        self._test_filter_by_field(
            self.book.summary_hash,
            BOOK_DATA_UPDATE['summary']
        )

        self._test_filter_by_field(
            self.book.description_hash,
            BOOK_DATA_UPDATE['description']
        )

        self._test_filter_by_field(
            self.book.credits_hash,
            BOOK_DATA_UPDATE['credits']
        )

        self._test_filter_by_field(
            self.book.synopsis_hash,
            BOOK_DATA_UPDATE['synopsis'].strip()
        )


if __name__ == '__main__':
    unittest.main()



"""
Fields.
"""

from .md5 import *


__title__ = 'strawberry.fields'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2018 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'MD5Field',
)



"""
Additional fields for(ever) Django.
"""

__title__ = 'django-strawberry'
__version__ = '0.1.1'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2018 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('default_app_config',)

default_app_config = 'strawberry.apps.Config'



"""
Apps.
"""

from django.apps import AppConfig

__title__ = 'strawberry.apps'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2018 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('Config',)


class Config(AppConfig):
    """Config."""

    name = 'strawberry'
    label = 'strawberry'



import logging

from django.db.models import CharField
from six import string_types
from .helpers import generate_md5_hash, get_prepopulated_value

__title__ = 'strawberry.fields'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2018 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'MD5Field',
)

LOGGER = logging.getLogger(__name__)


class MD5Field(CharField):
    """MD5 field."""

    def __init__(self, *args, **kwargs):
        kwargs['max_length'] = kwargs.get('max_length', 50)

        # Auto-populated md5 hash is not editable unless told so
        self.populate_from = kwargs.pop('populate_from', None)
        if self.populate_from:
            kwargs.setdefault('editable', False)

        # Strip whitespace
        self.strip_whitespace = kwargs.pop('strip_whitespace', True)

        # Unique_with value can be string or tuple
        self.unique_with = kwargs.pop('unique_with', ())
        if isinstance(self.unique_with, string_types):
            self.unique_with = (self.unique_with,)

        if self.unique_with:
            # We will do "manual" granular check below
            kwargs['unique'] = False

        # Set db_index=True unless it's been set manually.
        if 'db_index' not in kwargs:
            kwargs['db_index'] = True

        # When using model inheritance, set manager to search for matching
        # md5 hash values.
        self.manager = kwargs.pop('manager', None)
        self.manager_name = kwargs.pop('manager_name', None)

        self.always_update = kwargs.pop('always_update', True)

        super(MD5Field, self).__init__(*args, **kwargs)

    def deconstruct(self):
        """Deconstruct.

        Returns:
            `tuple`
        """
        name, path, args, kwargs = super(MD5Field, self).deconstruct()

        if self.max_length == 50:
            kwargs.pop('max_length', None)

        if self.populate_from is not None:
            kwargs['populate_from'] = self.populate_from
            if self.editable is not False:
                kwargs['editable'] = self.editable

        if self.unique_with != ():
            kwargs['unique_with'] = self.unique_with
            kwargs.pop('unique', None)

        kwargs.pop('db_index', None)

        if self.manager is not None:
            kwargs['manager'] = self.manager

        if self.manager_name is not None:
            kwargs['manager_name'] = self.manager_name

        if self.always_update:
            kwargs['always_update'] = self.always_update

        if 'manager' in kwargs:
            del kwargs['manager']

        return name, path, args, kwargs

    def pre_save(self, instance, add):
        """Pre-save.

        Args:
            instance:
            add:

        Returns:
            `str`
        """
        # Get currently entered md5 hash value
        value = self.value_from_object(instance)

        if self.manager is not None:
            manager = self.manager
        elif self.manager_name is not None:
            manager = getattr(self.model, self.manager_name)
        else:
            manager = None

        # Auto-populate
        if self.always_update or (self.populate_from and not value):
            value = get_prepopulated_value(self, instance)

            # pragma: nocover
            if __debug__ and not value and not self.blank:
                LOGGER.debug(
                    'Failed to populate md5 hash {}.{} from {}'.format(
                        instance._meta.object_name,
                        self.name,
                        self.populate_from
                    )
                )

        if value:
            if self.strip_whitespace:
                value = value.strip()
            # else:
            #     import pytest; pytest.set_trace()

            hash_value = generate_md5_hash(value)
        else:
            hash_value = None

            # if not self.blank:
            #     hash_value = instance._meta.model_name
            # elif not self.null:
            #     hash_value = ''

            # Ensure the hash_value is unique (if required)
            # if self.unique or self.unique_with:
            #     # TODO

            assert hash_value, 'value is filled before saving'

        # Make the updated md5 hash available as instance attribute
        setattr(instance, self.name, hash_value)

        return hash_value



import os

from setuptools import find_packages, setup

version = '0.1.1'

try:
    readme = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()
except:
    readme = ''

install_requires = [
    'six>=1.9',
    'django-nine>=0.1.10',
]

extras_require = []

tests_require = [
    'factory_boy',
    'fake-factory',
    'pytest',
    'pytest-django',
    'pytest-cov',
    'tox',
]

setup(
    name='django-strawberry',
    version=version,
    description="Additional fields for(ever) Django.",
    long_description=readme,
    classifiers=[
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Environment :: Web Environment",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
        "License :: OSI Approved :: GNU Lesser General Public License v2 or "
        "later (LGPLv2+)",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Development Status :: 4 - Beta",
    ],
    keywords="django, fields",
    author='Artur Barseghyan',
    author_email='artur.barseghyan@gmail.com',
    url='https://github.com/barseghyanartur/django-strawberry/',
    package_dir={'': 'src'},
    packages=find_packages(where='./src'),
    license='GPL 2.0/LGPL 2.1',
    install_requires=(install_requires + extras_require),
    tests_require=tests_require,
    include_package_data=True,
)
